///<reference path="../Scripts/linq.d.ts" />

module CodeCompass {

    export interface IInitable {
        init();
        isDisposed: bool;
    }

    export interface IDisposable {
        dispose();
        isDisposed: bool;
    }
}

module Common {

    export interface IHTMLObject {
        id: string;
        className: string;
    }

    export module Ensure {
        function isStrict(obj: any, func: (obj: any) => bool, propertyName?: string, message?: string) {
            if (!func(obj)) {
                var msg: string = "";
                if (!isBlank(propertyName))
                    msg += "[" + propertyName + "]: ";
                if (!isBlank(message))
                    msg += message;

                var c: any = console;
                c.trace();

                throw msg;
            }
            return Ensure;
        }

        export function defined(obj: any, propertyName?: string = "", message?: string = "Property cannot be undefined or null!"): Ensure {
            return isStrict(obj, o => isDefined(o) && isNotNull(o), propertyName, message);
        }

        export function notBlank(text: string, propertyName?: string, message?: string): Ensure {
            return isStrict(text, (obj) => !isBlank(obj), propertyName, message);
        }

        export function hasValue(obj: any) {
            return Ensure.isDefined(obj) && Ensure.isNotNull(obj);
        }

        export function isNotNull(obj: any) {
            return !Ensure.isNull(obj);
        }

        export function isNull(obj: any) {
            return obj === null;
        }

        export function isDefined(obj: any): bool {
            return obj !== undefined;
        }

        export function isUndefined(obj: any): bool {
            return !Ensure.isDefined(obj);
        }

        export function isFunction(obj: any): bool {
            return typeof (obj) === "function";
        }

        export function isBlank(text: string): bool {
            return !hasValue(text) || text === "";
        }

        export function toIntervallum(num: number, min: number = 0, max: number = 1) {
            Ensure.defined(min)
                  .defined(max)
                  .defined(num);

            if (num < min)
                num = min;
            else if (num > max)
                num = max;

            return num;
        }
    }

    export module Error {
    }

    export interface ISafeElement {
        has(...property: string[]): bool;
        tryGet(property: string, func: (value: any, object: any) => void ): ISafeElement;
        getObject(): any;
    }

    class SafeElement implements ISafeElement {
        constructor(object: any) {
            this._object = object;
        }

        private _object: any = null;

        getObject() { return this._object; }

        has(...property: string[]) {
            return Enumerable.From(property).All((value) => Ensure.hasValue(this._object[value]));
        }

        apply(func: (value: any) => any): ISafeElement {
            return new SafeElement(func(this._object));
        }

        tryGet(property: string, func: (value: any, sender?: any) => void ) {
            if (this.has(property))
                func(this._object[property], this._object);
            return this;
        }
    }

    export function Safe(object: any): ISafeElement {
        Ensure.defined(object);
        return new SafeElement(object);
    }

    export interface ILetElement {
        be(func: (value: any) => any): ILetElement;
        value: any;
        default(value: any);
    }

    class LetElement implements ILetElement {
        private _object: any;

        constructor(object) {
            this._object = object;
        }

        default(value: any) {
            var result = this._object;
            if (Ensure.isNull(result) || Ensure.isUndefined(result))
                result = value;
            return new LetElement(result);
        }

        be(func: (value: any) => any): ILetElement {
            return new LetElement(func(this._object));
        }

        get value() { return this._object; }
    }

    class LazyLetElement implements ILetElement {
        private _object: any;
        private _func: (value: any) => any;

        constructor(object, func: (value: any) => any = x => x) {
            this._object = object;
            this._func = func;
        }

        default(value: any) {
            return new LazyLetElement(this._object, x => {
                var result = this._func(x);
                if (Ensure.isNull(result) || Ensure.isUndefined(result))
                    result = value;
                return result;
            });
        }

        be(func: (value: any) => any): ILetElement {
            return new LazyLetElement(this._object, value => func(this._func(value)));
        }

        get value() { return this._func(this._object); }
    }

    export function Let(object: any, lazy: bool = false): ILetElement {
        if (lazy)
            return new LazyLetElement(object);
        else
            return new LetElement(object);
    }

    export function getClass(object: any) {
        return object.constructor;
    }

    export interface IFunctionObject {
        push(...items: any[]): any;
        pop(): any;
        clear(): any;
        (...params: any[]): any;
    };

    export var FunctionObject = function (...funcs: Function[]): IFunctionObject {
        /*if (!(this instanceof FunctionObject)) {
            return new FunctionObject(funcs);
        }*/

        var _functionList = funcs || [];

        var that: any = function (params) {
            var results = [];
            for (var ind in _functionList) {
                results.push(_functionList[ind](params));
            }
            return results;
        };

        that.elements = _functionList;

        that.push = function (...items: any[]) {
            for (var ind in items) {
                _functionList.push(items[ind]);
            }
            return that;
        };

        that.pop = function () {
            return _functionList.pop();
        };

        that.clear = function () {
            _functionList = [];
            return that;
        };

        return that;
    };

    ///Elkszt egy jQuery stlus property-t (nem az Object.defineProperty-t hasznlja)
    export var property = function (options: { get?: () => any; set?: (any) => any; default?: () => any; auto?: bool; that?: any; }) {
        if (!options)
            return undefined;

        if (options.auto) {
            var _value = options.default ? options.default() : undefined;
            return function (value?: any) {
                if (!value)
                    return _value;
                else {
                    _value = value;
                    return options.that;
                }
            };
        } else {
            return function (value?: any) {
                if (!value && options.get ) {
                    if (!options.get() && options.default)
                        options.set(options.default());
                    return options.get();
                }
                else if (options.set )
                    return options.set(value) || options.that;
            };
        }
    };
}

var Let = Common.Let;
var Ensure = Common.Ensure;
var Safe = Common.Safe;
var FunctionObject = Common.FunctionObject;
