///<reference path="Common.ts" />

module CodeCompass.Drawings {

    //Vszont reprezentl interfsz
    export interface ICanvas extends Common.IHTMLObject {
        width?: number; //Vszon szlessge
        height?: number; //Vszon magassga
    }

    //Sznt reprezentl osztly
    export class Color {
        private _R: number;
        private _G: number;
        private _B: number;
        private _A: number;

        //A szn R(ed), B(lue), G(reen) s A(lpha) rtkeit llthatjuk be vele
        get R(): number { return this._R; }
        get G(): number { return this._G; }
        get B(): number { return this._B; }
        get A(): number { return this._A; }

        constructor (red: number, green: number, blue: number, alpha: number = 1) {
            this._R = Ensure.toIntervallum(red, 0, 255);
            this._G = Ensure.toIntervallum(green, 0, 255);
            this._B = Ensure.toIntervallum(blue, 0, 255);
            this._A = Ensure.toIntervallum(alpha, 0, 1);
        }

        //Szn sszekeverse ms sznnel
        add(color: Color): Color {
            return new Color(this.R + color.R, this.G + color.G, this.B + color.B, this.A + color.A);
        }

        //Szn konvertlsa CSS kdd
        toCSS(): string {
            return "rgba(" + [this.R, this.G, this.B, this.A].join(",") + ")";
        }

        //Hexadecimilis rtkk val generlsa a sznnek
        //TODO: fix hex code generator
        toHex(): string {
            var hexCodes = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "a", "b", "c", "d", "e", "f"];
            var result: string = "#";

            [this.R, this.G, this.B, this.A * 255].forEach((value, index, array) => {
                if (value == 0)
                    result += "00";
                while (value != 0) {
                    result += hexCodes[value % 16];
                    value = Math.floor(value / 16);
                }
            });

            return result;
        }
    }

    //Skbeli pontot reprezentl interfsz
    export interface IPoint2D {
        x: number; //pont X koordintja
        y: number; //pont Y koordintja
        add? (point: IPoint2D): IPoint2D; //kt pont sszeadsrt felels metdus
        distance? (point: IPoint2D): number; //kt pont tvolsgnak kiszmtsrt felels metdus
        negate? (): IPoint2D; //pont neglst vgz metdus
    }

    //Mretet reprezentl interfsz
    export interface ISize {
        width: number; //mret szlessge
        height: number; //mret magassga
    }

    //Mretet megvalst osztly
    export class Size implements ISize {
        constructor (public width: number, public height: number) { }

        //Statikus adattag: Nulla mretet visszaad tulajdonsg
        static get Zero(): Size {
            return new Size(0, 0);
        }
    }
    //Skbeli pontot reprezentl osztly
    export class Point2D implements IPoint2D {
        private _x: number;
        private _y: number;
        //X, Y koordintt adja vissza
        get x() { return this._x; }
        get y() { return this._y; }

        constructor (x: number = 0, y: number = 0) {
            Ensure.defined(x, "x: number (Point2D)");
            Ensure.defined(y, "y: number (Point2D)");
            this._x = x;
            this._y = y;
        }

        //Statikus metdus, amivel knnyen kszthetnk j pontot
        static create(x: number = 0, y: number = 0) { return new Point2D(x, y); }

        //Visszatr az origo-val
        static get Origo(): Point2D {
            return new Point2D(0, 0);
        }

        //Kt pont sszeadsrt felels fggvny
        add(point: IPoint2D): Point2D {
            Ensure.defined(point, "point", "Point cannot be null or undefined!");
            return new Point2D(this.x + point.x, this.y + point.y);
        }

        //Kt pont tvolsgnak kiszmtsrt felels fggvny
        distance(point: IPoint2D = Point2D.Origo): number {
            Ensure.defined(point, "point: IPoint2D (Point2D)", "Point cannot be null or undefined!");
            var sqr = (x: number) => x * x;
            return Math.sqrt(sqr(this.x - point.x) + sqr(this.y - point.y));
        }

        //Pont neglsrt felels fggvny
        negate(): Point2D { return new Point2D(-this.x, -this.y); }
    }

    //Relatv koordintt reprezentl osztly
    export class RelativePoint2D extends Point2D {
        constructor (x: number = 0, y: number = 0,
                     //This point can be an absolute and relative point.
                     public isAbsolute = false,
                     public horizontalAlign: HorizontalAlign = HorizontalAlign.left,
                     public verticalAlign: VerticalAlign = VerticalAlign.top) {
            super(x, y);
        }
    }

    //Relatv mretet reprezentl osztly
    export class RelativeSize extends Size {
        constructor (public width: number = 0,
                     public height: number = 0,
                     public isAbsolute = false) {
            super(width, height);
        }

        static get Zero(): ISize {
            return new RelativeSize(0, 0);
        }
    }

    //Vonalstlust reprezentl intefsz
    export interface IStroke {
        color: Color;
        size: number;
        linecap?: LineCap;
        linejoin?: LineJoin;
        miterlimit?: number;
    }
    //Vzszintes elhelyezkedst reprezentl felsoroltpus
    export enum HorizontalAlign { left, center, right }
    //Fggleges elhelyezkedst reprezentl felsoroltpus
    export enum VerticalAlign { top, center, bottom }

    //Nyl vge szimblumot reprezentl felsoroltpus
    export enum ArrowEnd { classic, block, open, oval, diamond, none };

    //Nyl kalapjt reprezentl felsoroltpus
    export enum LineCap { butt, square, round }
    export enum LineJoin { bevel, round, miter }
}

//Grafikai primitvek intefszeit tartalmaz modul
module CodeCompass.Drawings.Primitives {

    //Grafikai objektumot reprezentl interfsz
    export interface IGraphicalObject {
        canvas: ICanvas;
        location: IPoint2D;
        size: ISize;
        visible: bool;
    }

    //Szveget reprezentl interfsz
    export interface IText extends IGraphicalObject {
        text: string;
        fontFamily: string;
        fontSize: number;
        foreground: Color;
        background?: Color;
        horizontalAlign: HorizontalAlign;
    }

    //Vonalat reprezentl interfsz
    export interface ILine extends IGraphicalObject {
        stroke: IStroke;
    }

    //Skbeli vonalat reprezentl interfsz
    export interface ILine2D extends ILine { }

    //Alakzatot reprezentl interfsz
    export interface IShape extends IGraphicalObject {
        border: IStroke;
        fill: Color;
    }

    //Logikai csoportot reprezentl interfsz
    export interface IGroup extends IGraphicalObject {
        children: IGraphicalObject[];
        clear();
        add(...children: IGraphicalObject[]): IGroup;
    }

    //Tglalapot reprezentl interfsz
    export interface IRectange extends IShape {

    }

    //Ellipszist reprezentl interfsz
    export interface IEllipse extends IShape {

    }

    //Polygon-t reprezentl interfsz
    export interface IPolygon extends IShape {
        points: IPoint2D[];
    }

    export interface IClickable {
        click: Function;
    }
}

module CodeCompass.Drawings {
    //Statikus osztly, ami elredefinilt szneket tartalmaz
    export class Colors {
        static get AliceBlue(): Color { return new Color(240, 248, 255, 1); }
        static get AntiqueWhite(): Color { return new Color(250, 235, 215, 1); }
        static get Aqua(): Color { return new Color(0, 255, 255, 1); }
        static get Aquamarine(): Color { return new Color(127, 255, 212, 1); }
        static get Azure(): Color { return new Color(240, 255, 255, 1); }
        static get Beige(): Color { return new Color(245, 245, 220, 1); }
        static get Bisque(): Color { return new Color(255, 228, 196, 1); }
        static get Black(): Color { return new Color(0, 0, 0, 1); }
        static get BlanchedAlmond(): Color { return new Color(255, 235, 205, 1); }
        static get Blue(): Color { return new Color(0, 0, 255, 1); }
        static get BlueViolet(): Color { return new Color(138, 43, 226, 1); }
        static get Brown(): Color { return new Color(165, 42, 42, 1); }
        static get BurlyWood(): Color { return new Color(222, 184, 135, 1); }
        static get CadetBlue(): Color { return new Color(95, 158, 160, 1); }
        static get Chartreuse(): Color { return new Color(127, 255, 0, 1); }
        static get Chocolate(): Color { return new Color(210, 105, 30, 1); }
        static get Coral(): Color { return new Color(255, 127, 80, 1); }
        static get CornflowerBlue(): Color { return new Color(100, 149, 237, 1); }
        static get Cornsilk(): Color { return new Color(255, 248, 220, 1); }
        static get Crimson(): Color { return new Color(220, 20, 60, 1); }
        static get Cyan(): Color { return new Color(0, 255, 255, 1); }
        static get DarkBlue(): Color { return new Color(0, 0, 139, 1); }
        static get DarkCyan(): Color { return new Color(0, 139, 139, 1); }
        static get DarkGoldenrod(): Color { return new Color(184, 134, 11, 1); }
        static get DarkGray(): Color { return new Color(169, 169, 169, 1); }
        static get DarkGreen(): Color { return new Color(0, 100, 0, 1); }
        static get DarkKhaki(): Color { return new Color(189, 183, 107, 1); }
        static get DarkMagenta(): Color { return new Color(139, 0, 139, 1); }
        static get DarkOliveGreen(): Color { return new Color(85, 107, 47, 1); }
        static get DarkOrange(): Color { return new Color(255, 140, 0, 1); }
        static get DarkOrchid(): Color { return new Color(153, 50, 204, 1); }
        static get DarkRed(): Color { return new Color(139, 0, 0, 1); }
        static get DarkSalmon(): Color { return new Color(233, 150, 122, 1); }
        static get DarkSeaGreen(): Color { return new Color(143, 188, 143, 1); }
        static get DarkSlateBlue(): Color { return new Color(72, 61, 139, 1); }
        static get DarkSlateGray(): Color { return new Color(47, 79, 79, 1); }
        static get DarkTurquoise(): Color { return new Color(0, 206, 209, 1); }
        static get DarkViolet(): Color { return new Color(148, 0, 211, 1); }
        static get DeepPink(): Color { return new Color(255, 20, 147, 1); }
        static get DeepSkyBlue(): Color { return new Color(0, 191, 255, 1); }
        static get DimGray(): Color { return new Color(105, 105, 105, 1); }
        static get DodgerBlue(): Color { return new Color(30, 144, 255, 1); }
        static get Firebrick(): Color { return new Color(178, 34, 34, 1); }
        static get FloralWhite(): Color { return new Color(255, 250, 240, 1); }
        static get ForestGreen(): Color { return new Color(34, 139, 34, 1); }
        static get Fuchsia(): Color { return new Color(255, 0, 255, 1); }
        static get Gainsboro(): Color { return new Color(220, 220, 220, 1); }
        static get GhostWhite(): Color { return new Color(248, 248, 255, 1); }
        static get Gold(): Color { return new Color(255, 215, 0, 1); }
        static get Goldenrod(): Color { return new Color(218, 165, 32, 1); }
        static get Gray(): Color { return new Color(128, 128, 128, 1); }
        static get Green(): Color { return new Color(0, 128, 0, 1); }
        static get GreenYellow(): Color { return new Color(173, 255, 47, 1); }
        static get Honeydew(): Color { return new Color(240, 255, 240, 1); }
        static get HotPink(): Color { return new Color(255, 105, 180, 1); }
        static get IndianRed(): Color { return new Color(205, 92, 92, 1); }
        static get Indigo(): Color { return new Color(75, 0, 130, 1); }
        static get Ivory(): Color { return new Color(255, 255, 240, 1); }
        static get Khaki(): Color { return new Color(240, 230, 140, 1); }
        static get Lavender(): Color { return new Color(230, 230, 250, 1); }
        static get LavenderBlush(): Color { return new Color(255, 240, 245, 1); }
        static get LawnGreen(): Color { return new Color(124, 252, 0, 1); }
        static get LemonChiffon(): Color { return new Color(255, 250, 205, 1); }
        static get LightBlue(): Color { return new Color(173, 216, 230, 1); }
        static get LightCoral(): Color { return new Color(240, 128, 128, 1); }
        static get LightCyan(): Color { return new Color(224, 255, 255, 1); }
        static get LightGoldenrodYellow(): Color { return new Color(250, 250, 210, 1); }
        static get LightGray(): Color { return new Color(211, 211, 211, 1); }
        static get LightGreen(): Color { return new Color(144, 238, 144, 1); }
        static get LightPink(): Color { return new Color(255, 182, 193, 1); }
        static get LightSalmon(): Color { return new Color(255, 160, 122, 1); }
        static get LightSeaGreen(): Color { return new Color(32, 178, 170, 1); }
        static get LightSkyBlue(): Color { return new Color(135, 206, 250, 1); }
        static get LightSlateGray(): Color { return new Color(119, 136, 153, 1); }
        static get LightSteelBlue(): Color { return new Color(176, 196, 222, 1); }
        static get LightYellow(): Color { return new Color(255, 255, 224, 1); }
        static get Lime(): Color { return new Color(0, 255, 0, 1); }
        static get LimeGreen(): Color { return new Color(50, 205, 50, 1); }
        static get Linen(): Color { return new Color(250, 240, 230, 1); }
        static get Magenta(): Color { return new Color(255, 0, 255, 1); }
        static get Maroon(): Color { return new Color(128, 0, 0, 1); }
        static get MediumAquamarine(): Color { return new Color(102, 205, 170, 1); }
        static get MediumBlue(): Color { return new Color(0, 0, 205, 1); }
        static get MediumOrchid(): Color { return new Color(186, 85, 211, 1); }
        static get MediumPurple(): Color { return new Color(147, 112, 219, 1); }
        static get MediumSeaGreen(): Color { return new Color(60, 179, 113, 1); }
        static get MediumSlateBlue(): Color { return new Color(123, 104, 238, 1); }
        static get MediumSpringGreen(): Color { return new Color(0, 250, 154, 1); }
        static get MediumTurquoise(): Color { return new Color(72, 209, 204, 1); }
        static get MediumVioletRed(): Color { return new Color(199, 21, 133, 1); }
        static get MidnightBlue(): Color { return new Color(25, 25, 112, 1); }
        static get MintCream(): Color { return new Color(245, 255, 250, 1); }
        static get MistyRose(): Color { return new Color(255, 228, 225, 1); }
        static get Moccasin(): Color { return new Color(255, 228, 181, 1); }
        static get NavajoWhite(): Color { return new Color(255, 222, 173, 1); }
        static get Navy(): Color { return new Color(0, 0, 128, 1); }
        static get OldLace(): Color { return new Color(253, 245, 230, 1); }
        static get Olive(): Color { return new Color(128, 128, 0, 1); }
        static get OliveDrab(): Color { return new Color(107, 142, 35, 1); }
        static get Orange(): Color { return new Color(255, 165, 0, 1); }
        static get OrangeRed(): Color { return new Color(255, 69, 0, 1); }
        static get Orchid(): Color { return new Color(218, 112, 214, 1); }
        static get PaleGoldenrod(): Color { return new Color(238, 232, 170, 1); }
        static get PaleGreen(): Color { return new Color(152, 251, 152, 1); }
        static get PaleTurquoise(): Color { return new Color(175, 238, 238, 1); }
        static get PaleVioletRed(): Color { return new Color(219, 112, 147, 1); }
        static get PapayaWhip(): Color { return new Color(255, 239, 213, 1); }
        static get PeachPuff(): Color { return new Color(255, 218, 185, 1); }
        static get Peru(): Color { return new Color(205, 133, 63, 1); }
        static get Pink(): Color { return new Color(255, 192, 203, 1); }
        static get Plum(): Color { return new Color(221, 160, 221, 1); }
        static get PowderBlue(): Color { return new Color(176, 224, 230, 1); }
        static get Purple(): Color { return new Color(128, 0, 128, 1); }
        static get Red(): Color { return new Color(255, 0, 0, 1); }
        static get RosyBrown(): Color { return new Color(188, 143, 143, 1); }
        static get RoyalBlue(): Color { return new Color(65, 105, 225, 1); }
        static get SaddleBrown(): Color { return new Color(139, 69, 19, 1); }
        static get Salmon(): Color { return new Color(250, 128, 114, 1); }
        static get SandyBrown(): Color { return new Color(244, 164, 96, 1); }
        static get SeaGreen(): Color { return new Color(46, 139, 87, 1); }
        static get SeaShell(): Color { return new Color(255, 245, 238, 1); }
        static get Sienna(): Color { return new Color(160, 82, 45, 1); }
        static get Silver(): Color { return new Color(192, 192, 192, 1); }
        static get SkyBlue(): Color { return new Color(135, 206, 235, 1); }
        static get SlateBlue(): Color { return new Color(106, 90, 205, 1); }
        static get SlateGray(): Color { return new Color(112, 128, 144, 1); }
        static get Snow(): Color { return new Color(255, 250, 250, 1); }
        static get SpringGreen(): Color { return new Color(0, 255, 127, 1); }
        static get SteelBlue(): Color { return new Color(70, 130, 180, 1); }
        static get Tan(): Color { return new Color(210, 180, 140, 1); }
        static get Teal(): Color { return new Color(0, 128, 128, 1); }
        static get Thistle(): Color { return new Color(216, 191, 216, 1); }
        static get Tomato(): Color { return new Color(255, 99, 71, 1); }
        static get Transparent(): Color { return new Color(255, 255, 255, 0); }
        static get Turquoise(): Color { return new Color(64, 224, 208, 1); }
        static get Violet(): Color { return new Color(238, 130, 238, 1); }
        static get Wheat(): Color { return new Color(245, 222, 179, 1); }
        static get White(): Color { return new Color(255, 255, 255, 1); }
        static get WhiteSmoke(): Color { return new Color(245, 245, 245, 1); }
        static get Yellow(): Color { return new Color(255, 255, 0, 1); }
        static get YellowGreen(): Color { return new Color(154, 205, 50, 1); }
    }
}