///<reference path="Common.ts" />
///<reference path="Drawings.ts" />
///<reference path="DrawingsRaphael.ts" />
///<reference path="../Scripts/linq.d.ts" />
///<reference path="Reflection.ts" />

module CodeCompass.Floyd.Diagram {
    var drawings = CodeCompass.Drawings;                //referencia a CodeCompass.Drawings modulra
    var Point2D = drawings.Point2D;                     //referencia a drawings.Point2D osztlyra
    var Size = drawings.Size;                           //referencia a drawings.Size osztlyra
    var primitives = Drawings.Raphael2D.Primitives;     //referencia a Drawings.Raphael2D.Primitives modulra
    var utilities = Drawings.Raphael2D.Utilities;       //referencia a Drawings.Raphael2D.Utilities modulra
    var Colors = drawings.Colors;                       //referencia a drawings.Colors osztlyra
    var toPoint = utilities.Size.toPoint;               //referencia a utilities.Size.toPoint fggvnyre
    var toSize = utilities.Point2D.toSize;              //referencia a utilities.Point2D.toSize fggvnyre
    var getClass = Common.getClass;                     //referencia a Common.getClass fggvnyre


    //Interface az IFloydDocument-re 
    export interface IFloydDocument {
        children: IFloydElement[];  //gyerekek lekrdezse
        start: IFloydStartElement; //a floyd grf kezd eleme
        end: IFloydEndElement;  //a floyd grf utols eleme
    }
    
    //egy floyd elemet reprezentl interfsz
    export interface IFloydElement {
        document: IFloydDocument;
        next: IFloydElement;
        previous: IFloydElement;
    }

    //Egyszer utastst reprezentl interfsz, ami az IFloydElement-bl rkldik
    export interface IFloydCommand extends IFloydElement {
        command: string;
    }

    //egyszer szekvencit reprezentl interfsz
    export interface IFloydBlock extends IFloydElement {
        children: IFloydElement[];
    }

    //egyszer ciklust reprezentl interfsz
    export interface IFloydLoop extends IFloydElement {
        condition: string;
        core: IFloydElement;
    }

    //egyszer elgazst reprezentl interfsz
    export interface IFloydCondition extends IFloydElement {
        condition: string;
        trueCase: IFloydElement;
        falseCase: IFloydElement;
    }

    //floyd diagram start szimbolumt reprezentl interfsz
    export interface IFloydStartElement extends IFloydElement {

    }

    //floyd diagram end szimbolumt reprezentl interfsz
    export interface IFloydEndElement extends IFloydElement {

    }

    //Floyd diagramot grfjt reprezentl osztly
    export class FloydGraph {
        //privt adattag: cscsokat trol mtrix
        private nodes: number[][];
        //privt adattag: grf nagysgt trol mret
        private _graphSize: Drawings.ISize;

        //a fggvny egy koordinta s mret alapjn, meghatrozza a Bellmann-Floyd algoritmushoz szksges mtrixot 
        private getNodes(location: Drawings.IPoint2D, size: Drawings.ISize, func: (node: number, location?: Drawings.IPoint2D) => number) {
            Ensure.defined(location)
                  .defined(size)
                  .defined(func);

            for (var i = Math.max(location.x, 0); i < Math.min(location.x + size.width, this._graphSize.width); ++i) {
                for (var j = Math.max(location.y, 0); j < Math.min(location.y + size.height, this._graphSize.height); ++j) {
                    this.nodes[j][i] = func(this.nodes[j][i], new Drawings.Point2D(i, j));
                }
            }
            //lncolhatsg miatt visszatr nmagval
            return this;
        }

        //Inicializlja a grf cscsait
        public initNodes(size: Drawings.ISize) {
            Ensure.defined(size)
                  .defined(size.width)
                  .defined(size.height);

            //size = new Size(100, 100);
            this._graphSize = size;
            this.nodes = [];

            for (var i = 0; i < size.width; ++i) {
                this.nodes.push([]);
                for (var j = 0; j < size.height; ++j) {
                    this.nodes[this.nodes.length - 1].push(1);
                }
            }
            return this;
        }

        //Bellman-Ford algortmus alapjn meghatrozza a nyilak koordintit 
        public planRoute(start: Drawings.IPoint2D, end: Drawings.IPoint2D) {
            //ellenrizzk, hogy definilva vannak-e a start s az end
            Ensure.defined(start)
                  .defined(end);

            var distance = [];
            var parent: Drawings.IPoint2D[][] = [];
            var isChanged = true;

            for (var i = 0; i < this._graphSize.width; ++i) {
                distance.push([]);
                parent.push([]);
                for (var j = 0; j < this._graphSize.height; ++j) {
                    distance[i].push(Number.POSITIVE_INFINITY);
                    parent[i].push(null);
                }
            }

            distance[start.y][start.x] = 0;

            var checkEdge = (u: Drawings.IPoint2D, v: Drawings.IPoint2D) => {
                if (v.x < 0 || v.x >= this._graphSize.width || v.y < 0 || v.y >= this._graphSize.height)
                    return;

                if (distance[v.y][v.x] > distance[u.y][u.x] + this.nodes[u.y][u.x]) {
                    distance[v.y][v.x] = distance[u.y][u.x] + this.nodes[u.y][u.x];
                    parent[v.y][v.x] = { x: u.x, y: u.y };
                    isChanged = true;
                }
            };
            for (var k = 0; k <= this._graphSize.width * this._graphSize.height - 1 && isChanged; ++k) {
                isChanged = false;
                for (var i = 0; i < this._graphSize.width; ++i) {
                    for (var j = 0; j < this._graphSize.height; ++j) {
                        [{ x: i - 1, y: j },
                         { x: i + 1, y: j },
                         { x: i, y: j - 1 },
                         { x: i, y: j + 1 }].forEach(value => {
                             var u = { x: i, y: j };
                             checkEdge(u, value);
                         });
                    }
                }
            }

            var route = [end];
            var p: Drawings.IPoint2D = parent[end.y][end.x];
            this.addObject(end, new Size(1, 1), 100);

            while (p !== null) {
                this.addObject(p, new Size(1, 1), 100);
                route.push(p);
                p = parent[p.y][p.x];
            }

            return route.reverse();
        }

        //Objektum trlse a grfbl
        public removeObject(location: Drawings.IPoint2D, size: Drawings.ISize) {
            this.getNodes(location, size, (node: number) => 1);
            return this;
        }

        //Objektum hozzadsa a grfhoz
        public addObject(location: Drawings.IPoint2D, size: Drawings.ISize, weight: number = 10000) {
            this.getNodes(location, size, (node: number) => weight);
            return this;
        }
    }

    //Floyd dokumentum reprezentcija
    export class FloydDocument implements IFloydDocument, IDisposable, IInitable {
        //privt adattag: rajzolst segt canvas
        private _canvas: Drawings.Raphael2D.Canvas;
        //privt adattag: floyd objektumokat tartalmaz tmb
        private _children: FloydElement[];
        //Floyd diagram start elemt tartalmaz adattag
        private _start: FloydStartElement;
        //Floyd diagram end elemt tartalmaz adattag
        private _end: IFloydEndElement;
        //Dokumentum nagysga
        private _size: Drawings.Size;
        //rajzolsra szksges vszon nagysgt jelzi
        private _canvasSize: Drawings.Size;
        //Grfot reprezentl adattag
        private _graph: FloydGraph;
        //Logikai rtk, ami igazzal tr vissza, ha mr felszabadtottk a memribl az adattagjait
        private _isDisposed: bool = false;
        //Nyilakat, relcikat trol tmb
        private _arrows: Drawings.Raphael2D.Primitives.MultiLine2D[];

        //Igazzal tr vissza, ha az objektum megsemmistsre vr a memriban
        get isDisposed() { return this._isDisposed; }

        //Visszatr a grf gyermekeivel
        get children(): FloydElement[] { return this._children; }

        //Visszatr a start szimblummal
        get start(): FloydStartElement { return this._start; }

        //Visszatr az end szimblummal
        get end(): IFloydEndElement { return this._end; }

        //Visszatr a grf nagysgval
        get size() { return this._size; }

        //Visszatr azzal a Raphael-es canvas-sal amire rajzolunk
        public getCanvas() { return this._canvas; }

        //Relatv koordintt hatroz meg a canvas-os
        public fromPoint(location: Drawings.Point2D) {
            return Point2D.create(Math.floor(location.x / (this._canvasSize.width / this.size.width)),
                                  Math.floor(location.y / (this._canvasSize.height / this.size.height)));
        }

        //abszolt koordintt hatroz meg a canvas-on
        public getPoint(location: Drawings.Point2D) {
            return Point2D.create(location.x * (this._canvasSize.width / this.size.width),
                                  location.y * (this._canvasSize.height / this.size.height));
        }
        
        //Visszatr a grf nagysgval
        public getSize(size: Drawings.Size) {
            return new Size(size.width * (this._canvasSize.width / this.size.width),
                            size.height * (this._canvasSize.height / this.size.height));
        }

        //trli az sszes gyerekt
        private clear() {
            this._children = [];
            return this;
        }

        //a gyerekelemek sszekapcsolst vgz fggvny, ami a gyerekek kz inicializlja a nyilakat
        private connectChildren(element: FloydElement) {
            Ensure.defined(element);

            if (getClass(element) === FloydCondition) {
                var condition = <FloydCondition>element;
                this.connectElement(condition, condition.trueCase, 0);
                this.connectElement(condition, condition.falseCase, 1);
                this.connectChildren(condition.trueCase).connectChildren(condition.falseCase);
            }
            else if (getClass(element) === FloydLoop) {
                var loop = <FloydLoop>element;
                if (Ensure.hasValue(loop.core)) {
                    this.connectChildren(loop.core);
                    this.connectElement(loop, loop.core, 1);
                    this.connectElement(loop.core, loop, 0, 1);
                }
            }
            else if (getClass(element) === FloydBlock) {
                var block = <FloydBlock>element;
                block.children.forEach(child => {
                    Ensure.defined(child);
                    this.connectChildren(child);
                    if (Ensure.hasValue(child.next)) {
                        this.connectElement(child, child.next);
                    }
                });
            }

            return this;
        }

        //sszekapcsol kt floyd elemet
        private connectElement(start: FloydElement, end: FloydElement, startIndex: number = 0, endIndex: number = 0) {
            Ensure.defined(start)
                  .defined(end);

            if (getClass(start) === FloydCondition && getClass(end) === FloydConditionEndElement) {
                var condition = <FloydCondition>start;
                this.connectElement(condition.trueCase, end);
                this.connectElement(condition.falseCase, end);
            }
            else if (getClass(start) === FloydBlock) {
                var block = <FloydBlock>start;
                this.connectElement(Enumerable.From(block.children).Last(), end, startIndex, endIndex);
            }
            else if (getClass(end) === FloydBlock) {
                var block = <FloydBlock>end;
                this.connectElement(start, Enumerable.From(block.children).First(), startIndex, endIndex);
            }
            else {
                var startPoint = start.outputPoints[startIndex];
                var endPoint = end.inputPoints[endIndex];

                startPoint = startPoint.add(start.element.location);
                endPoint = endPoint.add(end.element.location);

                var route = this._graph.planRoute(this.fromPoint(startPoint), this.fromPoint(endPoint));
                var arrow = new Drawings.Raphael2D.Primitives.MultiLine2D(this._canvas, startPoint, endPoint);
                arrow.arrowEnd = Drawings.ArrowEnd.block;
                arrow.stroke.size = 3;

                this._arrows.push(arrow);

                var points = Enumerable.From(route)
                                       //.Skip(1)
                                       .TakeExceptLast()
                                       .Select((value: Drawings.IPoint2D) => this.getPoint(new Drawings.Point2D(value.x, value.y)))
                                       .Select((value: Drawings.IPoint2D) => Drawings.Point2D.create(
                                                                                       value.x + this._canvasSize.width / this.size.width / 2.0,
                                                                                       value.y + this._canvasSize.height / this.size.height / 2.0))
                                       .ToArray();

                points.unshift(startPoint);
                points.push(endPoint);

                arrow.setPoints(points);
            }
            return this;
        }

        //elhelyezi a floyd elemeket a vsznon
        private arrangeElement(element: FloydElement, location: Drawings.Point2D = Drawings.Point2D.create(10, 1)): Drawings.Size {
            Ensure.defined(element);
            var returnSize = new Drawings.Size(0, 0);

            if (getClass(element) === FloydCondition) {
                var condition = <FloydCondition>element;

                var trueCaseSize = this.arrangeElement(condition.trueCase, location.add(Drawings.Point2D.create(0, condition.size.height + 2)));
                var falseCaseSize = this.arrangeElement(condition.falseCase,
                                                        location.add(Drawings.Point2D.create(trueCaseSize.width + 2, condition.size.height + 2)));

                condition.location = Drawings.Point2D.create(location.x + Math.floor((trueCaseSize.width + falseCaseSize.width + 2) / 2.0 - condition.size.width / 2.0), location.y);

                returnSize = new Drawings.Size(trueCaseSize.width + falseCaseSize.width + 2,
                                               Math.max(trueCaseSize.height, falseCaseSize.height) + condition.size.height);
            }
            else if (getClass(element) === FloydBlock) {
                var block = <FloydBlock>element;
                block.children.forEach(child => {
                    var size = this.arrangeElement(child, location);
                    location = location.add(Drawings.Point2D.create(0, size.height + 2));
                    returnSize = new Drawings.Size(Math.max(returnSize.width, size.width), returnSize.height + size.height + 2);
                });
            }
            else if (getClass(element) === FloydLoop) {
                var loop = <FloydLoop>element;
                loop.location = location;
                var size = this.arrangeElement(loop.core, location.add(Drawings.Point2D.create(0, loop.size.height + 2)));
                returnSize = new Drawings.Size(Math.max(loop.size.width, size.width), loop.size.height + size.height);
            }
            else {
                element.location = location;
                returnSize = element.size;
            }

            if (getClass(element) !== FloydBlock)
                this._graph.addObject(element.location, element.size);

            return returnSize;
        }

        //Automatikusan elhelyezi a gyerekelemeket
        public arrangeLayout() {
            this._graph = new FloydGraph();
            this._graph.initNodes(this._size);
            this._arrows = [];

            var block = new FloydBlock(this, this._children);

            this.arrangeElement(block);
            this.connectChildren(block);

            return this;
        }

        //inicializlja a dokumentumot
        public init() {
            Ensure.defined(this._children);
            this._children.forEach(e => e.init());
        }

        //"Felszabadtja" a dokumentumot
        public dispose() {
            if (!this._isDisposed && Ensure.isDefined(this._children)) {
                this._children.forEach(e => e.dispose());
                this._isDisposed = true;
            }
        }

        //FloydDocument konstruktora
        constructor(elementName: string = "canvas") {
            Ensure.notBlank(elementName, "elementName: string (FloydDocument)");
            this._canvasSize = new Size(1000, 1000);
            this._canvas = new Drawings.Raphael2D.Canvas(elementName, this._canvasSize.width, this._canvasSize.height);
            this._size = new Size(50, 50);

            this._graph = new FloydGraph();
            this._graph.initNodes(this._size);

            this._children = [];
        }
    }

    //Floyd objektumot reprezentl osztly
    export class FloydElement implements IFloydElement, IDisposable, IInitable {
        private _element: Drawings.Raphael2D.Primitives.GraphicalObject;
        private _document: FloydDocument;
        private _location: Drawings.Point2D;
        private _inputPoints: Drawings.Point2D[];
        private _outputPoints: Drawings.Point2D[];
        private _size: Drawings.Size;
        private _isDisposed: bool;

        //visszatr az adott grafikai objektummal
        get element() { return this._element; }

        //visszatr a befelmen kapcsoldsi pontok egy tmbjvel
        get inputPoints() { return this._inputPoints; }
        //visszatr a kifelmen kapcsoldsi pontok egy tmbjvel
        get outputPoints() { return this._outputPoints; }
        get size() { return this._size; }

        //Visszatr az elem pozicijval
        get location() { return this._location; }

        //Belltja egy elem pozicijt
        set location(value: Drawings.Point2D) {
            Ensure.defined(value);
            this._location = value;
            this.element.location = this._document.getPoint(value);
        }

        //az elem inicializlsrt felels fggvny
        public init() {
            this.dispose();
            this._isDisposed = false;
        }

        //Az elem "felszabadtsrt" felels fggvny
        public dispose() {
            if (Ensure.isDefined(this._element)) {
                this._element.dispose();
                this._isDisposed = true;
            }
        }

        //visszatr a hozz rendelt Floyd dokumentummal
        get document() { return this._document; }
        //Igazzal tr vissza, ha az objektum felszabadtsra vr
        get isDisposed() { return this._isDisposed; }

        //Konstruktor
        constructor(document: FloydDocument, public previous: FloydElement = null, public next: FloydElement = null) {
            Ensure.defined(document, "document: FloydDocument (FloydElement)");
            this._document = document;
            this._inputPoints = [];
            this._outputPoints = [];
            this._size = new Drawings.Size(5, 3);
            this._isDisposed = true;
        }
    }

    //Floyd start elemt reprezentl osztly
    export class FloydStartElement extends FloydElement implements IFloydStartElement {
        private _element: Drawings.Raphael2D.Primitives.GraphicalObject;

        public init() {
            super.init();

            var group = new Drawings.Raphael2D.Primitives.Group(this.document.getCanvas(), Point2D.Origo, this.document.getSize(this.size));
            var ellipse = new primitives.Ellipse(this.document.getCanvas(), this.document.getPoint(Point2D.create(2.5, 1.5)), new Drawings.RelativeSize(1, 1));
            ellipse.border.color = Colors.CornflowerBlue;
            ellipse.fill = Colors.White;
            ellipse.border.size = 3;
            var text = new primitives.Text(this.document.getCanvas(), "start", Point2D.create(group.size.width / 2.0, group.size.height / 2.0 - 7.5), 15, "Verdana", Colors.DarkBlue);
            text.horizontalAlign = Drawings.HorizontalAlign.center;
            this._element = group.add(ellipse, text);

            this.inputPoints.push(Drawings.Point2D.create(this.element.size.width / 2.0, 0));
            this.outputPoints.push(Drawings.Point2D.create(this.element.size.width / 2.0, this.element.size.height));
        }

        constructor(document: FloydDocument) {
            super(document);
        }
    }

    //Floyd elgazst reprezentl osztly
    export class FloydConditionEndElement extends FloydElement implements IFloydEndElement {
        private _element: Drawings.Raphael2D.Primitives.GraphicalObject;

        public init() {
            super.init();
            var group = new Drawings.Raphael2D.Primitives.Group(this.document.getCanvas(), Point2D.Origo, this.document.getSize(this.size));
            var ellipse = new primitives.Ellipse(this.document.getCanvas(), Point2D.create(group.size.width / 2.0, group.size.height / 2.0 - 7.5), new Drawings.RelativeSize(0.9, 0.9));
            ellipse.border.color = Colors.CornflowerBlue;
            ellipse.fill = Colors.White;
            ellipse.border.size = 3;
            var text = new primitives.Text(this.document.getCanvas(), "condition\nend", Point2D.create(group.size.width / 2.0, group.size.height / 2.0 - 15), 15, "Verdana", Colors.DarkBlue);
            text.horizontalAlign = Drawings.HorizontalAlign.center;
            this._element = group.add(ellipse, text);

            this.inputPoints.push(Drawings.Point2D.create(this.element.size.width / 2.0, 0));
            this.outputPoints.push(Drawings.Point2D.create(this.element.size.width / 2.0, this.element.size.height - 7.5));
        }

        //Konstruktor
        constructor(document: FloydDocument) {
            super(document);
        }
    }

    //Floyd end elemt reprezentl osztly
    export class FloydEndElement extends FloydElement implements IFloydEndElement {
        private _element: Drawings.Raphael2D.Primitives.GraphicalObject;

        public init() {
            super.init();
            var group = new Drawings.Raphael2D.Primitives.Group(this.document.getCanvas(), Point2D.Origo, this.document.getSize(this.size));
            var ellipse = new primitives.Ellipse(this.document.getCanvas(), Point2D.create(group.size.width / 2.0, group.size.height / 2.0 - 7.5), new Drawings.RelativeSize(0.9, 0.9));
            ellipse.border.color = Colors.CornflowerBlue;
            ellipse.fill = Colors.White;
            ellipse.border.size = 3;
            var text = new primitives.Text(this.document.getCanvas(), "end", this.document.getPoint(Point2D.create(2, 1)).add(Point2D.create(0, -7.5)), 15, "Verdana", Colors.DarkBlue);
            text.horizontalAlign = Drawings.HorizontalAlign.center;
            this._element = group.add(ellipse, text);

            this.inputPoints.push(Drawings.Point2D.create(this.element.size.width / 2.0, 0));
            this.outputPoints.push(Drawings.Point2D.create(this.element.size.width / 2.0, this.element.size.height));
        }

        constructor(document: FloydDocument) {
            super(document);
        }
    }

    //Egyszer parancsot reprezentl osztly
    export class FloydCommand extends FloydElement implements IFloydCommand {
        private _element: Drawings.Raphael2D.Primitives.GraphicalObject;

        public init() {
            super.init();
            var group = new Drawings.Raphael2D.Primitives.Group(this.document.getCanvas(), Point2D.Origo, this.document.getSize(this.size));
            var rectangle = new primitives.Rectange(this.document.getCanvas(), Point2D.Origo, new Drawings.RelativeSize(1, 1));
            rectangle.border.color = Colors.CornflowerBlue;
            rectangle.fill = Colors.White;
            rectangle.border.size = 3;
            var text = new primitives.Text(this.document.getCanvas(), this.command, Point2D.create(group.size.width / 2.0, group.size.height / 2.0 - 7.5), 15, "Verdana", Colors.DarkBlue);
            text.horizontalAlign = Drawings.HorizontalAlign.center;
            this._element = group.add(rectangle, text);

            this.inputPoints.push(Drawings.Point2D.create(this.element.size.width / 2.0, 0));
            this.outputPoints.push(Drawings.Point2D.create(this.element.size.width / 2.0, this.element.size.height));
        }

        constructor(document: FloydDocument, public command: string) {
            super(document);
        }
    }

    //Blokkot reprezentl osztly
    //TODO: element must be a Group!!!
    export class FloydBlock extends FloydElement implements IFloydBlock {
        private _children: FloydElement[];
        //private _element: Drawings.Raphael2D.Primitives.GraphicalObject;

        get inputPoints(): Drawings.Point2D[] {
            var firstElement = <FloydElement>Enumerable.From(this._children).First();

            if (Ensure.isDefined(firstElement)) {
                return firstElement.inputPoints;
            }

            return [];
        }

        get outputPoints(): Drawings.Point2D[] {
            var lastElement = <FloydElement>Enumerable.From(this._children).Last();

            if (Ensure.isDefined(lastElement)) {
                return lastElement.outputPoints;
            }

            return [];
        }

        /**
        *   It gets children of FloydBlock.
        */
        get children() { return this._children; }

        public dispose() {
            if (Ensure.isDefined(this._children))
                this._children.forEach(e => e.dispose());
        }

        public init() {
            super.init();
            if (Ensure.isDefined(this._children)) {
                this._children.forEach(e => e.init());
            }

            this.inputPoints.push(Drawings.Point2D.create(0, 0));
            this.outputPoints.push(Drawings.Point2D.create(0, 0));
        }

        /**
        *   Constructor of FloydBlock
        *   @param {FloydDocument} document Current Floyd document
        *   @param {FloydElement[]} children It sets children of FloydBlock
        */
        constructor(document: FloydDocument, children: FloydElement[] = []) {
            super(document);
            this._children = children;
            //this._element = new Drawings.Raphael2D.Primitives.Group(document.getCanvas(), Drawings.Point2D.Origo, new Drawings.Size(1, 1));
        }
    }

    //Elgazst reprezentl osztly
    export class FloydCondition extends FloydElement implements IFloydCondition {
        private _trueCase: FloydElement;
        private _falseCase: FloydElement;
        private _element: Drawings.Raphael2D.Primitives.GraphicalObject;

        //Visszatr az igaz ggal
        get trueCase() { return this._trueCase; }

        //Az igaz gat lehet belltani a segtsgvel
        set trueCase(value: FloydElement) {
            Ensure.defined(value);
            if (Ensure.hasValue(this._trueCase))
                this._trueCase.dispose();

            this._trueCase = value;
            this._trueCase.init();
        }

        //Visszatr a hamis ggal
        get falseCase() { return this._falseCase; }

        //A hamis gat lehet belltani vele 
        set falseCase(value: FloydElement) {
            Ensure.defined(value);
            if (Ensure.hasValue(this._falseCase))
                this._falseCase.dispose();

            this._falseCase = value;
            this._falseCase.init();
        }

        public init() {
            super.init();
            var group = new Drawings.Raphael2D.Primitives.Group(this.document.getCanvas(), Point2D.Origo, this.document.getSize(this.size));
            var rhombus = new primitives.Rhombus(this.document.getCanvas(), Point2D.Origo, new Drawings.RelativeSize(1, 1));
            rhombus.border.color = Colors.CornflowerBlue;
            rhombus.fill = Colors.White;
            rhombus.border.size = 3;
            var text = new primitives.Text(this.document.getCanvas(), this.condition, Point2D.create(group.size.width / 2.0, group.size.height / 2.0 - 7.5), 15, "Verdana", Colors.DarkBlue);
            text.horizontalAlign = Drawings.HorizontalAlign.center;
            this._element = group.add(rhombus, text);

            this.inputPoints.push(Drawings.Point2D.create(this.element.size.width / 2.0, 0));

            this.outputPoints.push(Drawings.Point2D.create(-1, this.element.size.height / 2.0));
            this.outputPoints.push(Drawings.Point2D.create(this.element.size.width, this.element.size.height / 2.0));
        }

        constructor(document: FloydDocument, public condition: string) {
            super(document);

        }
    }

    //Ciklust reprezentl osztly
    export class FloydLoop extends FloydElement implements IFloydLoop {
        private _element: Drawings.Raphael2D.Primitives.GraphicalObject;
        private _core: FloydElement;

        //Visszatr a ciklusmaggal
        get core() { return this._core; }
        //A ciklusmagot lehet belltani
        set core(value: FloydElement) {
            Ensure.defined(value);
            if (Ensure.hasValue(this._core))
                this._core.dispose();

            this._core = value;
            this._core.init();
        }

        public init() {
            super.init();
            var group = new Drawings.Raphael2D.Primitives.Group(this.document.getCanvas(), Point2D.Origo, this.document.getSize(this.size));
            var ellipse = new primitives.Ellipse(this.document.getCanvas(), this.document.getPoint(Point2D.create(2.5, 1.5)), new Drawings.RelativeSize(1, 1));
            ellipse.border.color = Colors.CornflowerBlue;
            ellipse.fill = Colors.White;
            ellipse.border.size = 3;
            var text = new primitives.Text(this.document.getCanvas(), this.condition, Point2D.create(group.size.width / 2.0, group.size.height / 2.0 - 7.5), 15, "Verdana", Colors.DarkBlue);
            text.horizontalAlign = Drawings.HorizontalAlign.center;
            this._element = group.add(ellipse, text);

            this.inputPoints.push(Drawings.Point2D.create(this.element.size.width / 2.0, 0));
            this.inputPoints.push(Drawings.Point2D.create(0, this.element.size.height / 2.0));
            this.outputPoints.push(Drawings.Point2D.create(this.element.size.width, this.element.size.height / 2.0));
            this.outputPoints.push(Drawings.Point2D.create(this.element.size.width / 2.0, this.element.size.height));
        }

        constructor(document: FloydDocument, public condition: string) {
            super(document);
        }
    }

    //Floyd-beli JSON objektumot reprezentl interfsz
    export interface IFloydJSON {
        id?: string;
        type?: string;
        attributes?: {
            cond?: string;
            init?: string;
            inc?: string;
            step?: string;
        };
        block?: IFloydJSON[];
        elseBlock?: IFloydJSON[];
        text?: string;
    }

    //Floyd elemeket gyrt osztly
    export class FloydElementFactory {
        private _document: FloydDocument;
        private _children: FloydElement[];

        //Visszatr a gyerek elemekkel
        get children() { return this._children; }
        //Visszatr a hozz prostott FloydDocument-el
        get document() { return this._document; }

        constructor(document: FloydDocument) {
            this._document = document;
            this._children = [];
        }

        //Frissti az elkszlsre vr elemeket
        private updateElement(element: FloydElement) {
            Ensure.defined(element);
            element.previous = this.lastElement(v => { v.next = element; });
            return element;
        }

        //trli a gyerekeket
        clear() {
            this._children = [];
        }

        //Blokkot kszt a gyerekelemekbl
        toBlock() {
            return new FloydBlock(this._document, this._children);
        }

        //Visszatr az utols gyerekkel s a neki paramterknt tadott fggvny segtsgvel vgrehajt valami mveletet rajta
        private lastElement(func: (element: FloydElement) => void = null) {
            var last = Enumerable.From(this._children).LastOrDefault(undefined);
            if (Ensure.isDefined(last) && Ensure.isDefined(func))
                func(last);
            return last;
        }

        //Start elemet kszt
        start() {
            this._children.push(new FloydStartElement(this._document));
            return this;
        }

        //End elemet kszt
        end() {
            this._children.push(this.updateElement(new FloydEndElement(this._document)));
            return this;
        }

        //Egyszer parancsot kszt
        command(text: string) {
            var _command = new FloydCommand(this._document, text);
            this._children.push(this.updateElement(_command));
            return this;
        }

        //ciklust kszt fggvny
        loop(condition: string, core: (factory: FloydElementFactory) => FloydElementFactory) {
            var factory = new FloydElementFactory(this._document);
            var loop = new FloydLoop(this._document, condition);
            loop.core = core(factory).toBlock();
            //loop.core.next = loop;
            this._children.push(this.updateElement(loop));
            return this;
        }

        //elgazst kszt fggvny
        condition(condition: string, trueCase: (factory: FloydElementFactory) => FloydElementFactory, falseCase: (factory: FloydElementFactory) => FloydElementFactory) {
            var factory = new FloydElementFactory(this._document);
            var ifThenElse = new FloydCondition(this._document, condition);
            ifThenElse.trueCase = trueCase(factory).toBlock();
            factory = new FloydElementFactory(this._document);
            ifThenElse.falseCase = falseCase(factory).toBlock();

            var ifThenElseEnd = new FloydConditionEndElement(this._document);

            this._children.push(this.updateElement(ifThenElse));
            this._children.push(this.updateElement(ifThenElseEnd));

            return this;
        }

        //blokkot kszt fggvny
        block(children: (factory: FloydElementFactory) => FloydElementFactory) {
            var factory = new FloydElementFactory(this._document);
            var block = children(factory).toBlock();
            this._children.push(this.updateElement(block));
            return this;
        }

        //JSON-bl generl floyd diagramot
        fromJSON(json: IFloydJSON[]) {
            Ensure.defined(json);

            json.forEach((value: IFloydJSON, index) => {

                var assert = function (condition: bool, errorMsg: string) {
                    if (condition)
                        console.error(errorMsg);
                };

                assert(!Safe(value).has("id"), "Error: Invalid format of file! Missing id at " + value.type + " type element!");

                if (!Safe(value).has("type")) {
                    console.error("Error: Invalid format of file! Missing type at element with id " + value.id.toString() + "!");
                    return this;
                }
                else if (value.type !== "for" && value.type !== "do" && value.type !== "if" && value.type !== "command") {
                    console.error("Error: invalid format of file! Unknown type at element with id " + value.id.toString() + "!");
                    return this;
                }

                var errorMsg = "";
                var addError = function (condition: bool, msg: string) {
                    if (condition)
                        errorMsg += msg + "\n";
                }

                switch (value.type) {
                    case "command":
                        if (!Safe(value).has("text")) {
                            console.error("Error: Invalid format of file! Missing text at element with id " + value.id.toString() + "!");
                        }
                        else if (getClass(value.text) !== String) {
                            console.error("Error: Invalid format of file! The text attribute is not a string at element with id " + value.id.toString() + "!");
                        }
                        else
                            return this.command(value.text);
                        break;
                    case "do": case "for":
                        addError(!Safe(value).has("attributes"), "Missing attributes!");
                        addError(!Safe(value).has("block"), "Missing block!");
                        addError(!Safe(value.attributes).has("cond"), "Missing condition of loop!");
                        addError(getClass(value.block) !== Array, "Invalid format of file!");
                        addError(getClass(value.attributes) !== Object, "Invalid format of file!");
                        addError(getClass(value.attributes.cond) !== String, "Invalid format of file!");

                        return this.loop(value.attributes.cond, core => core.fromJSON(value.block));

                    case "if":
                        addError(!Safe(value).has("block"), "Missing block!");
                        addError(!Safe(value).has("elseBlock"), "Missing else block!");
                        addError(!Safe(value).has("attributes"), "Missing attributes!");
                        addError(!Safe(value.attributes).has("cond"), "Missing condition!");

                        addError(getClass(value.attributes) !== Object, "Wrong format of attributes!");
                        addError(getClass(value.block) !== Array, "Wrong format of block!");
                        addError(getClass(value.elseBlock) !== Array, "Wrong format of else block!");
                        addError(getClass(value.attributes.cond) !== String, "Wrong format of condition!");

                        if (errorMsg !== "") {
                            console.error("Error: Invalid format of file: element id: " + value.id.toString() + "\n" + errorMsg);
                        }

                        return this.condition(value.attributes.cond,
                            trueCase => trueCase.fromJSON(value.block),
                            falseCase => falseCase.fromJSON(value.elseBlock));
                }
            });

            return this;
        }
    }
}